# zope
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Acquisition import aq_base

# silva
from Products.Silva.interfaces import IContainer
from Products.Silva.interfaces import IPublishable
from Products.Silva.Folder import Folder
from Products.Silva.helpers import add_and_edit
from Products.Silva import mangle

# local
from interfaces import ILocalContainer, ILocalSilvaObject
from LocalSilvaObject import LocalSilvaObject
from util import AcceptLanguageFake

class LocalContainerMixin(LocalSilvaObject):
    """ a mixin which should be implemented by all locale aware containers
    """

    manage_options = Folder.manage_options + LocalSilvaObject.manage_options

    def __init__(self, id , title, language):
        # nothing to do here (yet)
        LocalSilvaObject.__init__(self, id, title, language)


    def get_supported_languages(self):
        # return the available languages of the parent:
        return self.aq_inner.aq_parent.get_available_languages()

    def create_language_variant(self, new_language, old_language=None):
        LocalContainerMixin.inheritedAttribute('create_language_variant') \
                                 (self, new_language, old_language)
        default = self.get_default()
        if default is not None and ILocalSilvaObject.isImplementedBy(default):
            # the new language variant should not exist for the default ...
            default.create_language_variant(new_language, old_language)
    

    # hm, this forwards everything to the "default",
    # which is most probably not the right thing ...
    # especially in case of custom container views
    # if would be more orthogonal to have a "language" parameter
    # to the "is_published" ...
    def _get_public_languages1(self):
        default = self.get_default()
        if default is not None:
            return default.get_public_languages()
        return ()


    def _get_approved_languages1(self):
        default = self.get_default()
        if default is not None:
            return default.get_approved_languages()
        return ()


    # ... which can be faked by the following implementation:
    def _get_public_languages2(self):
        old_accept = self.REQUEST.other['AcceptLanguage']
        self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(None)
        langs = []
        for lang in self.get_available_languages():
            dummy_accept.lang = lang
            if self.is_published():
               langs.append(lang)
        self.REQUEST.other['AcceptLanguage'] = old_accept
        return langs

    def _get_approved_languages2(self):
        old_accept = self.REQUEST.other['AcceptLanguage']
        self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(None)
        langs = []
        for lang in self.get_available_languages():
            dummy_accept.lang = lang
            if self.is_approved():
               langs.append(lang)
        self.REQUEST.other['AcceptLanguage'] = old_accept
        return langs


    # select the variant to be used
    get_public_languages   = _get_public_languages2
    get_approved_languages = _get_approved_languages2

    # XXX to_folder / to_publication does not work ...
    def to_folder(self):
        raise RuntimeError, 'converting to folder is not supported'

    def to_publication(self):
        raise RuntimeError, 'converting to publication is not supported'

icon='www/silvafolder.gif'
addable_priority=-1.9
class LocalFolder(LocalContainerMixin, Folder):
    """ a silva folder being able to contain content objects with locale variants
    """

    __implements__ = (ILocalContainer, )
    meta_type = 'Silva Local Folder'

    def __init__(self, id, title, language):
        assert isinstance(self,LocalFolder)
        assert isinstance(self,LocalSilvaObject)
        Folder.__init__(self, id)
        LocalContainerMixin.__init__(self, id, title, language)


InitializeClass(LocalFolder)

manage_addLocalFolderForm = PageTemplateFile("www/folderAdd", globals(),
                                        __name__='manage_addLocalFolderForm')

def manage_addLocalFolder(self, id, title, create_default=1, policy_name='Silva Local Document', REQUEST=None):
    """Add a LocalFolder."""
    if not mangle.Id(self, id).isValid():
        return
    language = self.get_selected_language()
    object = LocalFolder(id, title, language)
    self._setObject(id, object)
    object = getattr(self, id)
    object._setupAfterAdd()
    object.set_title(title)
    
    if create_default:
        policy = self.service_containerpolicy.getPolicy(policy_name)
        policy.createDefaultDocument(object, title)
    
    add_and_edit(self, id, REQUEST)
    return ''
