# zope
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# silva
from Products.Silva.Root import Root
from Products.Silva.Security import Security
from Products.Silva.helpers import add_and_edit
from Products.Silva import SilvaPermissions, mangle

# local
from interfaces import ILocalRoot
from LocalSilvaObject import LocalSilvaObject
from LocalFolder import LocalContainerMixin
from util import AcceptLanguageFake

class LocalRoot(LocalContainerMixin, Root):
    """ a silva root being able to contain content objects with locale variants
    """

    __implements__ = (ILocalRoot, )
    meta_type = 'Silva Local Root'
    
    inherited_manage_options = LocalContainerMixin.manage_options
    manage_options= (
        (inherited_manage_options[0],) +
        ({'label':'Services', 'action':'manage_services'},) +
        inherited_manage_options[1:]
        )
    
    security = ClassSecurityInfo()
    
    def __init__(self, id, title, language):
        #print self.__class__.__bases__
        assert isinstance(self,LocalRoot)
        assert isinstance(self,LocalSilvaObject)
        # XXX interestingly _this_ fails sometimes :-/
        assert isinstance(self,Security)
        Root.__init__(self, id)
        LocalContainerMixin.__init__(self, id, title, language)


    # implementation of ILocalRoot
    # ... is already implemented by LanguageManager, except for:

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_supported_languages')
    def get_supported_languages(self):
        # the root always supports all available languages (?)
        return self.get_available_languages()

    # hmm, set_default_language ???

    # notify localizer, too, if language added
    def add_language(self, language):
        """ """
        localizer = getattr(self, 'Localizer', None)
        if localizer is not None:
            localizer.add_language(language)
        LocalRoot.inheritedAttribute('add_language')(self, language)
        self._cretate_metadata_for_language(language, None)
        
InitializeClass(LocalRoot)


manage_addLocalRootForm = PageTemplateFile("www/localrootAdd", globals(),
                                           __name__='manage_addLocalRootForm')

def manage_addLocalRoot(self, id, title, languages, REQUEST=None):
    " Add a LocalRoot "
    if not mangle.Id(self, id).isValid():
        return
    if len(languages) == 0:
        raise 'Bad Request','must define at least one language'
    object = LocalRoot(id, title, languages[0])
    self._setObject(id, object)
    object = getattr(self, id)

    from Products.Silva import install as silva_install
    import install

    silva_install.installFromScratch(object)
    install.install(object)

    if getattr(object,'Localizer',None) is None:
        # no localizer: add one
        # XXX adding it in the silva root means the root
        # only works in a virtual host situation, where
        # the virtual host root it the silva root ...
        object.manage_addProduct['Localizer'].\
               manage_addLocalizer('Localizer for Silva', languages)

    # fix an issue with registering the extension to the "Extensions Service"
    object.service_view_registry.set_trees(object.service_extensions.get_installed_names())

    import LocalDocument
    addables = object.get_silva_addables_allowed_in_publication()
    addables.append(LocalDocument.LocalDocument.meta_type)
    object.set_silva_addables_allowed_in_publication(addables)

    object._setupAfterAdd()
    object.set_title(title)

    for lang in languages[1:]:
        object.add_language(lang)

    # XXX add a demo index ?

    add_and_edit(self, id, REQUEST)
    return 
