#
# this module defines a couple of helper methods
# which are possibly helpful to upgrade custom content types
# to be locale aware.
#
# it assumes the custom content types are already updated
# on the file system level to inherit from one of the
# LocalXXX classes, and only the content on the ZODB needs upgrade
#
# the methods itself needs to be called by some upgrade of the
# content objects (or a helper script); they do not hook them in
# by themselves magically or the like -- please do this by hand
#
# the contents are initialized with the "currently selected" language,
# so You hopefully have set this to the right value when upgrading ;-)
#
# XXX: these migration scripts have only been moderately tested with
# a Silva 0.9.1 setup and are *certainly* broken for the current release
# please use them only as an initial attempt to get a working migration;
# You will need to add heavy fixes

from Acquisition import aq_base
import zLOG
from LocalVersionedContent import LocaleVariant
from interfaces import ILocalSilvaObject, ILocalVersionedContent

def _is_already_migrated(obj):
    # guess its already upgraded
    # thus heuristic is probably wrong ...

    # if "_languages" is zero width, it is most propably the class variable
    # in LanguageManager ....
    return len(obj._languages) != 0


def migrate_local_silva_object(obj, force_upgrade = 0):
    # upgrade an object which implements LocalSilvaObject

    if not ILocalSilvaObject.isImplementedBy(obj):
        raise TypeError, 'object %s of type %s does not implement ILocalSilvaObject' % \
              (obj.absolute_url(), obj.meta_type)
    
    if not force_upgrade and _is_already_migrated(obj):
        return obj

    lang = obj.get_selected_language()
    if lang is None:
        raise TypeError, 'no default language selected'
    obj._languages = (lang,)
    obj._default_language = lang

    assert obj.get_default_language()==lang
    #zLOG.LOG('migration', -100, 'set title %s for language %s' % \
    #         ( getattr(obj,'_title', u'no title?').encode('ascii','replace'), lang )
    #         )
    # XXX title migration: works no longer;
    # should add metadata delegates instead
    if hasattr(aq_base(obj), 'is_default') and obj.is_default():
        obj.set_title(obj.get_container().get_title())
        # most probably does not have a title, but anyway:
        try:
            del obj._title
        except KeyError:
            pass
    else:
        obj.set_title(obj._title)
        del obj._title

    # checks if the title is migrated
    obj.get_title()
    getattr(obj, '_title_%s' % lang)

    return obj


def migrate_local_versioned_content(obj, force_upgrade = 0):

    if not ILocalVersionedContent.isImplementedBy(obj):
        raise TypeError, 'object %s of type %s does not implement ILocalVersionedContent' % \
              (obj.absolute_url(), obj.meta_type)

    if not force_upgrade and _is_already_migrated(obj):
        return obj

    migrate_local_silva_object(obj, force_upgrade)

    # zLOG.LOG('migration',-100,'upgrade object %s' % obj.get_title_html())
    lang = obj.get_selected_language()
    obj._setObject(lang, LocaleVariant(lang,title=u''))
    lang_object = getattr(obj, lang)
    # XXX setting a title for the LocalVariant is actually pointless; should be unused

    # determine all used version objects
    b = aq_base(obj)
    versions = [ v[0] for v in (b._previous_versions or [])]
    v = b._public_version
    if v[0] is not None:        
        versions.append(v[0])
    v = b._approved_version
    if v[0] is not None:        
        versions.append(v[0])
    v = b._unapproved_version
    if v[0] is not None:
        versions.append(v[0])

    #zLOG.LOG('migration',-100,'copy versions %s' % versions)
    versions = [ v for v in versions if hasattr(b, v) ]
    #zLOG.LOG('migration',-100,'copy versions %s' % versions)

    # copy over the version objects
    cp = obj.manage_cutObjects(versions)
    lang_object.manage_pasteObjects(cb_copy_data = cp)

    # copy over version info
    for attribute in ('_unapproved_version', '_approved_version', '_public_version', \
                      '_previous_versions', '_request_for_approval_info', '_version_count'):
        setattr(lang_object, attribute, getattr(b, attribute))
        try:
            delattr(b, attribute)
        except KeyError:
            # happens if class variable ...
            pass

    return obj
    
